/*
 * Copyright  2005 Sun Microsystems Inc. All rights reserved. Use is subject to license terms.
 *  
 */

/*
 * Created on Jun 7, 2005
 *
 */
package com.sunlabs.archsim.globals;

import java.io.File;
import java.io.IOException;
import java.util.Map;

import org.xml.sax.*;
import org.xml.sax.helpers.DefaultHandler; 

import com.sunlabs.archsim.engine.Director;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParserFactory;   
import javax.xml.parsers.SAXParser;

/**
 * @author ib27688
 *
 * Load the global settings.
 * For s hard-coded list of settings, if a setting value is
 * not provided, then fill in a default value.
 */
class GlobalsLoader extends DefaultHandler {
	
	public final static String SETTINGS= "settings";
	public final static String VALUE= "value";
	
	private Director director= Director.getInstance();
	
	/** Locator provided by the parser */
	private Locator locator;

	/** flag indicating that the root element has been encountered */
	private boolean inRoot= false;
	
	Map<String,String> values;
	
	void load(String fileName, Map<String,String> valueMap) {
		values= valueMap; 
		SAXParserFactory factory = SAXParserFactory.newInstance();
        // no DTD supplied, non validating parser used
		// factory.setValidating(true);
        //factory.setNamespaceAware(true);
        try {
            // Parse the input
            SAXParser saxParser = factory.newSAXParser();
            saxParser.parse( new File(fileName), this);

        } catch (SAXParseException spe) {
           // Error generated by the parser
           director.error("\n** Parsing error"
           		+ ", line " + spe.getLineNumber()
				+ ", uri " + spe.getSystemId()
				+ "   " + spe.getMessage() );

           // Use the contained exception, if any
           Exception  x = spe;
           if (spe.getException() != null)
               x = spe.getException();
           x.printStackTrace();
           System.exit(1);

        } catch (SAXException sxe) {
            // Error generated by this application
            // (or a parser-initialization error)
      
        		// first take a look at a message associated with
        		// 
        		String message= sxe.getMessage();
        		if (!message.equals("")) {
        			director.error(message);
        		}
        		
        		Exception  x = sxe;
        		if (sxe.getException() != null)
            		x = sxe.getException();
        		x.printStackTrace();
        		System.exit(1);

        } catch (ParserConfigurationException pce) {
            	// Parser with specified options can't be built
            	pce.printStackTrace();

        } catch (IOException ioe) {
        		// I/O error
        		ioe.printStackTrace();
        }		
        
	} // load

	/** 
	 * The callback that provides a Locator - used to get line numbers
	 * for error messages, for example.
	 */
    public void setDocumentLocator(Locator l) {
    		locator = l;
    }

	
	/**
	 *  prep work to be done at the beginning of the document 
	 */
	public void startDocument() throws SAXException {
		// do nothing
	} // startDocument
	
	/** 
	 * Finishing work to be done at the end of the XML file
	 */
	public void endDocument() throws SAXException {
		// reset inRoot
		inRoot= false;
	} // endDocument
	
	public void startElement (
			String namespaceURI,
			String lName, // local name
			String qName, // qualified name
			Attributes attrs) 
	throws SAXException {
		String eName = lName; // element name
	     if ("".equals(eName)) eName = qName; // namespaceAware = false
	     // if not the root element , record an operation
	     if (!inRoot && eName.equals(SETTINGS)) {
	     	// this is the root element, set the flag
	     	inRoot= true;
	     }
	     else if (!inRoot && !eName.equals(SETTINGS)) {
	     	throw new SAXException(locator.getLineNumber()
	     			+ ": ERROR: The root element should be " 
	     			+ SETTINGS + " and not " + eName);
	     }
	     else {
	     	// found a new element
	     	// the name of the element is the name of the parameter
	     	// now search for value in the attributes
	     	// ignore the element if the value is not found
	     	boolean valueFound= false;
	     	for (int i= 0; i < attrs.getLength() && !valueFound; i++) {
	     		String aName= attrs.getLocalName(i);
	     		if (aName.equals("")) aName= attrs.getQName(i);
	     		
	     		if (aName.equals(VALUE) ) {
	     			// found value, record it, exit the loop     			
	     			values.put(eName, attrs.getValue(i));
	     			valueFound= true;
	     		}
	     	} // for
	     	
	     } // if not root element
	    
	} // startElement
	
    public void endElement(String namespaceURI,
            String lName, // simple name
            String qName  // qualified name
           )
    throws SAXException {
    		// do nothing
    } // endElement
	
	
	public void characters(char buf[], int offset, int len)
	throws SAXException {
		// do nothing
	} // characters
	
    // treat validation errors as fatal
    public void error(SAXParseException e)
    throws SAXParseException
    {
        throw e;
    }

    // dump warnings too
    public void warning(SAXParseException err)
    throws SAXParseException
    {
        director.error("** Warning"
        		+ ", line " + err.getLineNumber()
            	+ ", uri " + err.getSystemId()
        		+ "   " + err.getMessage());
    } // Warning
    
    /** 
     * Print a warining.
     * @param msg warning message to be displayed.
     */
    protected void printWarning(String msg) {
        director.error("** Warning"
                + ", line " + locator.getLineNumber()
                + ", uri " + locator.getSystemId());
            director.error("   " + msg);   	
    } // msg
    
    
} // GlobalsLoader
